import { Button, ButtonGroup } from '@material-ui/core'
import MaxPowerIcon from 'assets/icons/ico/maxPower.svg'
import MinIcon from 'assets/icons/ico/minimum.svg'
import OffPeakHourIcon from 'assets/icons/ico/offPeakHour.svg'
import classNames from 'classnames'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import Loader from 'components/Loader/Loader'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/providers/I18n'
import { FluidType, TimeStep } from 'enums'
import {
  AggregatedEnedisMonthlyDataloads,
  EnedisMonthlyAnalysisData,
  FluidPrice,
  PerformanceIndicator,
} from 'models'
import React, { useCallback, useEffect, useMemo, useState } from 'react'
import ConsumptionService from 'services/consumption.service'
import EnedisMonthlyAnalysisDataService from 'services/enedisMonthlyAnalysisData.service'
import FluidPricesService from 'services/fluidsPrices.service'
import { useAppSelector } from 'store/hooks'
import { getFluidIcon } from 'utils/picto'
import { OffPeakHours } from 'utils/utils'
import ElecHalfHourChart from './ElecHalfHourChart'
import ElecInfoModal from './ElecInfoModal'
import './elecHalfHourMonthlyAnalysis.scss'

const ElecHalfHourLayout = ({ children }: { children: React.ReactNode }) => {
  const { t } = useI18n()
  return (
    <div className="special-elec-container">
      <StyledIcon
        className="elec-icon"
        icon={getFluidIcon(FluidType.ELECTRICITY)}
        size={42}
      />
      <div className="text-18-normal title">{t('special_elec.title')}</div>
      {children}
    </div>
  )
}

const ElecHalfHourMonthlyAnalysis = ({
  perfIndicator,
}: {
  perfIndicator: PerformanceIndicator
}) => {
  const { t } = useI18n()
  const client = useClient()
  const { analysisMonth } = useAppSelector(state => state.ecolyo.analysis)

  const [isWeekend, setIsWeekend] = useState<boolean>(true)
  const [isHalfHourActivated, setIsHalfHourActivated] = useState<boolean>(false)
  const [isLoading, setIsLoading] = useState<boolean>(true)
  const [monthDataloads, setMonthDataloads] =
    useState<AggregatedEnedisMonthlyDataloads>()
  const [enedisAnalysisValues, setEnedisAnalysisValues] =
    useState<EnedisMonthlyAnalysisData>()
  const [facturePercentage, setFacturePercentage] = useState<number>()
  const [elecPrice, setElecPrice] = useState<FluidPrice>()
  const [openInfoModal, setOpenInfoModal] = useState<boolean>(false)
  const [offPeakHours, setOffPeakHours] = useState<OffPeakHours[]>()

  /** EnedisMonthlyAnalysisDataService */
  const emas = useMemo(
    () => new EnedisMonthlyAnalysisDataService(client),
    [client]
  )

  const isDataFullyComplete = useCallback(
    (monthDataloads: AggregatedEnedisMonthlyDataloads | undefined) => {
      return (
        monthDataloads?.weekend &&
        monthDataloads.week &&
        monthDataloads.weekend[0]?.value !== null &&
        monthDataloads.week[0]?.value !== null
      )
    },
    []
  )

  const getPowerChart = useCallback((): JSX.Element => {
    if (monthDataloads && isDataFullyComplete(monthDataloads)) {
      return (
        <ElecHalfHourChart
          dataLoad={isWeekend ? monthDataloads.weekend : monthDataloads.week}
          isWeekend={isWeekend}
        />
      )
    }
    return <p className="text-20-bold no_data">{t('analysis.no_data')}</p>
  }, [isDataFullyComplete, isWeekend, monthDataloads, t])

  useEffect(() => {
    let subscribed = true
    async function fetchEnedisAnalysisData() {
      const cs = new ConsumptionService(client)
      const isHalfHourLoadActivated = await cs.checkDoctypeEntries(
        FluidType.ELECTRICITY,
        TimeStep.HALF_AN_HOUR
      )
      if (isHalfHourLoadActivated) {
        setIsHalfHourActivated(true)
        const aggregatedDate = analysisMonth.minus({ month: 1 })
        const data = await emas.getEnedisMonthlyAnalysisByDate(
          aggregatedDate.year,
          aggregatedDate.month
        )
        if (data?.length) {
          const aggregatedData = emas.aggregateValuesToDataLoad(data[0])
          setEnedisAnalysisValues(data[0])
          setMonthDataloads(aggregatedData)
          if (data[0].minimumLoad && perfIndicator.value) {
            const percentage = (data[0].minimumLoad / perfIndicator.value) * 100
            setFacturePercentage(percentage)
          }
        }
      }

      setIsLoading(false)
    }
    subscribed && fetchEnedisAnalysisData()

    return () => {
      subscribed = false
    }
  }, [analysisMonth, client, emas, perfIndicator])

  useEffect(() => {
    let subscribed = true
    const fluidsPricesService = new FluidPricesService(client)
    async function getAllLastPrices() {
      const price = await fluidsPricesService.getPrices(
        FluidType.ELECTRICITY,
        analysisMonth.minus({ month: 1 })
      )
      if (price) {
        setElecPrice(price)
      }
    }
    subscribed && getAllLastPrices()

    return () => {
      subscribed = false
    }
  }, [analysisMonth, client])

  useEffect(() => {
    let subscribed = true
    async function getOffPeakHours() {
      const offPeakHours = await emas.getOffPeakHours()
      if (offPeakHours) {
        setOffPeakHours(offPeakHours)
      }
    }
    subscribed && getOffPeakHours()

    return () => {
      subscribed = false
    }
  }, [emas])

  if (!isHalfHourActivated) {
    return (
      <ElecHalfHourLayout>
        <div className="activation-text text-16-normal">
          {t(`timestep.half_an_hour.analysis_waiting_data`)}
        </div>
      </ElecHalfHourLayout>
    )
  }

  if (isLoading) {
    return (
      <ElecHalfHourLayout>
        <Loader color="elec" />
      </ElecHalfHourLayout>
    )
  }

  return (
    <ElecHalfHourLayout>
      <div className="text-20-normal title">{t('special_elec.average')}</div>
      <ButtonGroup role="tablist" style={{ backgroundColor: 'var(--grey-0)' }}>
        <Button
          role="tab"
          className={classNames('btnTab', { active: !isWeekend })}
          onClick={() => setIsWeekend(false)}
        >
          {t('special_elec.week')}
        </Button>
        <Button
          role="tab"
          className={classNames('btnTab', { active: isWeekend })}
          onClick={() => setIsWeekend(true)}
        >
          {t('special_elec.weekend')}
        </Button>
      </ButtonGroup>

      {getPowerChart()}
      {enedisAnalysisValues && isDataFullyComplete(monthDataloads) && (
        <>
          <div className="elec-infos">
            <div className="container">
              <StyledIcon icon={MaxPowerIcon} size={40} className="minIcon" />
              <div className="text text-18-normal">
                <div className="row">
                  {t('special_elec.maxPower')}
                  {enedisAnalysisValues.maxPower !== 0 &&
                  enedisAnalysisValues.maxPower !== null ? (
                    <div className="value">
                      <div className="text-18-bold">
                        {enedisAnalysisValues.maxPower.toFixed(1)}
                      </div>
                      <span> kVA</span>
                    </div>
                  ) : (
                    <span>----</span>
                  )}
                </div>
              </div>
            </div>
            {enedisAnalysisValues?.offPeakHoursRatio != null && (
              <div className="container">
                <StyledIcon
                  icon={OffPeakHourIcon}
                  size={40}
                  className="minIcon"
                />
                <div className="text text-18-normal">
                  <div className="row">
                    {t('special_elec.offPeakHour')}
                    <div className="value">
                      <span className="text-18-bold">
                        {Math.round(
                          enedisAnalysisValues.offPeakHoursRatio * 100
                        )}
                      </span>
                      <span> %</span>
                    </div>
                  </div>
                </div>
              </div>
            )}
            <div className="container">
              <StyledIcon icon={MinIcon} size={40} className="minIcon" />
              <div className="text text-18-normal">
                {enedisAnalysisValues.minimumLoad !== 0 &&
                enedisAnalysisValues.minimumLoad !== null ? (
                  <>
                    <div className="row">
                      <div>{t('special_elec.min')}</div>
                      <div className="value">
                        <span className="text-18-bold">
                          {enedisAnalysisValues.minimumLoad.toFixed(1)}
                        </span>
                        <span> kWh</span>
                      </div>
                    </div>
                    <div className="row">
                      <div>{t('special_elec.percentage')}</div>
                      <div className="value">
                        <span className="text-18-bold">
                          {facturePercentage?.toFixed(0) || '---'}
                        </span>
                        <span> %</span>
                      </div>
                    </div>
                    <div className="row">
                      <div>{t('special_elec.price')}</div>
                      <div className="value">
                        <span className="text-18-bold">
                          {elecPrice?.price
                            ? (
                                enedisAnalysisValues.minimumLoad *
                                elecPrice.price
                              ).toFixed(1)
                            : '---'}
                        </span>
                        <span> €</span>
                      </div>
                    </div>
                  </>
                ) : (
                  <span>----</span>
                )}
              </div>
            </div>
          </div>

          <Button className="btnText" onClick={() => setOpenInfoModal(true)}>
            {t('special_elec.showModal')}
          </Button>
        </>
      )}

      <ElecInfoModal
        open={openInfoModal}
        offPeakHours={offPeakHours}
        handleCloseClick={() => setOpenInfoModal(false)}
      />
    </ElecHalfHourLayout>
  )
}

export default ElecHalfHourMonthlyAnalysis
