import { Collapse } from '@material-ui/core'
import Button from '@material-ui/core/Button'
import Dialog from '@material-ui/core/Dialog'
import CloseIcon from 'assets/icons/ico/close.svg'
import defaultIcon from 'assets/icons/visu/ecogesture/default.svg'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import StyledIconButton from 'components/CommonKit/IconButton/StyledIconButton'
import useExploration from 'components/Hooks/useExploration'
import { useI18n } from 'cozy-ui/transpiled/react/providers/I18n'
import { Ecogesture } from 'models'
import React, { useEffect, useState } from 'react'
import { useAppSelector } from 'store/hooks'
import { getFluidIcon } from 'utils/picto'
import { importIconById } from 'utils/utils'
import EfficiencyRating from '../EfficiencyRating/EfficiencyRating'
import './ecogestureModal.scss'

interface EcogestureModalProps {
  open: boolean
  ecogesture: Ecogesture
  isAction: boolean
  handleCloseClick: () => void
  selectEcogesture?: () => void
}

const EcogestureModal = ({
  open,
  ecogesture,
  isAction,
  handleCloseClick,
  selectEcogesture,
}: EcogestureModalProps) => {
  const { t } = useI18n()
  const { currentChallenge } = useAppSelector(state => state.ecolyo.challenge)
  const [ecogestureIcon, setEcogestureIcon] = useState('')
  const [showDetails, setShowDetails] = useState(false)

  const [, setValidExploration] = useExploration()

  useEffect(() => {
    async function handleEcogestureIcon() {
      const icon = await importIconById(ecogesture.id, 'ecogesture')
      setEcogestureIcon(icon || defaultIcon)
      if (currentChallenge?.exploration.ecogesture_id === ecogesture._id) {
        setValidExploration(currentChallenge.exploration.id)
      }
    }
    if (ecogesture) {
      handleEcogestureIcon()
    }
  }, [ecogesture, setValidExploration, currentChallenge])

  return (
    <Dialog
      open={open}
      onClose={handleCloseClick}
      aria-labelledby="accessibility-title"
      classes={{
        root: 'modal-root',
        paper: 'modal-paper no-padding',
      }}
    >
      <div id="accessibility-title">
        {isAction
          ? t('ecogesture_modal.accessibility.window_title_action')
          : t('ecogesture_modal.accessibility.window_title_ecogesture')}
      </div>
      <StyledIconButton
        icon={CloseIcon}
        onClick={handleCloseClick}
        aria-label={t('ecogesture_modal.accessibility.button_close')}
        className="modal-paper-close-button"
      />
      <div className="em-header text-14-normal-uppercase">
        {isAction
          ? t('ecogesture_modal.title_action')
          : t('ecogesture_modal.title_ecogesture')}
      </div>

      <div className="em-root ecogesture-modal">
        <div className="em-content">
          <div className="em-content-box-img">
            {ecogestureIcon && (
              <StyledIcon className="icon" icon={ecogestureIcon} size={140} />
            )}
          </div>
          <div className="em-title text-24-bold ">{ecogesture.shortName}</div>
          <div className="em-detail">
            <div className="em-detail-efficiency">
              <span className="em-efficiency">
                {t('ecogesture_modal.efficiency')}
              </span>
              <EfficiencyRating result={Math.round(ecogesture.efficiency)} />
            </div>
            <div className="em-picto-flow">
              {ecogesture.fluidTypes.map(fluid => (
                <StyledIcon
                  key={fluid}
                  className="em-pic-content"
                  icon={getFluidIcon(fluid)}
                  size={25}
                />
              ))}
            </div>
          </div>
          <div className="text-16-bold long-name">{ecogesture.longName}</div>
          {isAction ? (
            <Button
              aria-label={t(
                'ecogesture_modal.accessibility.button_select_action'
              )}
              className="btnPrimary"
              onClick={selectEcogesture}
            >
              {t('ecogesture_modal.select_action')}
            </Button>
          ) : (
            <>
              <Collapse in={showDetails}>
                <div className="em-description text-16-normal-150">
                  {ecogesture.longDescription}
                </div>
              </Collapse>
              <Button
                aria-label={t(
                  'ecogesture_modal.accessibility.button_see_more_detail'
                )}
                onClick={() => setShowDetails(prev => !prev)}
                className="btnSecondary"
              >
                {t(`ecogesture_modal.show_${showDetails ? 'less' : 'more'}`)}
              </Button>
            </>
          )}
        </div>
      </div>
    </Dialog>
  )
}

export default EcogestureModal
